
'Klaus D. Raudszus
'Raudszusklaus@aol.com

Option Strict On

Imports System.Security.Permissions
Imports System.Windows.Forms
Imports System.Drawing
Imports System.ComponentModel
Imports System.Environment
Imports System.IO
Imports FileInformation
Imports System.Management

<PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust"), _
ToolboxBitmap(GetType(DirectoriesBrowser))> _
Public Class DirectoriesBrowser
    Inherits TreeView

    Public Event FolderSelectionChanged(ByVal sender As [Object], ByVal e As EventArgs)
    Friend WithEvents ImageList1 As ImageList
    Private components As IContainer
    Protected m_ArrayListSpecialFolder As ArrayList
    Protected m_strSelectedPath As String
    Protected m_intImageIndex As Integer
    Protected m_intSelectedImageIndex As Integer = 8
    Protected IsInitialized As Boolean = False
    Protected IsSelecting As Boolean = False

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Public Sub New()
        InitializeComponent()
        Me.ImageList = ImageList1
        Me.ItemHeight = 18
        InitBrowser()
    End Sub

#Region " InitializeComponent "

    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Dim resources As System.Resources.ResourceManager = New System.Resources.ResourceManager(GetType(DirectoriesBrowser))
        Me.ImageList1 = New System.Windows.Forms.ImageList(Me.components)
        '
        'ImageList1
        '
        Me.ImageList1.ImageSize = New System.Drawing.Size(16, 16)
        Me.ImageList1.ImageStream = CType(resources.GetObject("ImageList1.ImageStream"), System.Windows.Forms.ImageListStreamer)
        Me.ImageList1.TransparentColor = System.Drawing.Color.Transparent

    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

#End Region

    <Browsable(True), _
    EditorBrowsable(EditorBrowsableState.Always), _
    Category("DirectoriesBrowserHandling"), _
    Description("Pfad kann eingeben werden, um nicht bei den Datentrgern anzufangen")> _
    Public Property Path() As String
        Get
            Return m_strSelectedPath
        End Get
        Set(ByVal Value As String)
            If Value Is Nothing Or Value = "" Then Return
            If Not IsInitialized Then
                m_strSelectedPath = Value
                Return
            End If
            Try
                If Not Directory.Exists(Value) Then Throw New _
                   ArgumentException("Path", _
                   "Das eingegebene Verzeichnis existiert nicht!")
                m_strSelectedPath = Value
            Catch
                Path = "" : Return
            End Try
            m_strSelectedPath = Value
            SetSelection(Value)
        End Set
    End Property
    'Inialisiert die 0-Knoten und ermittelt die Datentrger
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Public Sub InitBrowser()
        Me.Cursor = Cursors.WaitCursor
        ListSpecialFolder()
        Dim tn As TreeNode
        Dim mos As New ManagementObjectSearcher("SELECT * FROM " & "Win32_LogicalDisk")
        Dim moc As ManagementObjectCollection = mos.Get()
        Dim mo As ManagementObject
        For Each mo In moc
            Try
                Dim strDeviceID As String = mo.Properties("DeviceID").Value.ToString
                Dim strDriveTyp As Integer = Convert.ToInt32(mo.Properties("DriveType").Value)
                Dim strVolName As String = CStr(mo.Properties("VolumeName").Value)
                Dim strVolDesc As String = mo.Properties("Description").Value.ToString()
                Dim str As String
                If strVolName Is Nothing Then : str = strVolDesc
                Else : str = strVolName
                End If
                str = String.Format("{0} {1} {2} ({3})", " ", str, " ", strDeviceID)
                tn = AddNode(Me.Nodes, str, strDeviceID & "\")
                tn.ImageIndex = strDriveTyp
                tn.SelectedImageIndex = strDriveTyp
                AddNode(tn.Nodes, ".", ".")
            Catch ex As Exception
                Throw ex
            End Try
        Next
        IsInitialized = True
        Path = m_strSelectedPath
        Me.Cursor = Cursors.Default
    End Sub
    'Ermittelt bei Neuerstellung der Komponente die Spezialordner und speichert sie in einer Arrayliste
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Function ListSpecialFolder() As Boolean
        If m_ArrayListSpecialFolder Is Nothing Then
            m_ArrayListSpecialFolder = New ArrayList
        Else : m_ArrayListSpecialFolder.Clear()
        End If
        Dim strFolders() As String = System.Enum.GetNames(GetType(Environment.SpecialFolder))
        Dim intValue() As Integer = DirectCast(System.Enum.GetValues _
                                    (GetType(Environment.SpecialFolder)), Integer())
        Dim intCount As Integer = intValue.Length - 1
        For i As Integer = 0 To intCount
            Dim strArrayItem As String = CStr(strFolders(i)) & ";" & _
                     Environment.GetFolderPath(CType(intValue(i), Environment.SpecialFolder))
            m_ArrayListSpecialFolder.Add(strArrayItem)
        Next
        Return True
    End Function

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Sub SetSelection(ByVal path As String)
        Me.Cursor = Cursors.WaitCursor
        Me.BeginUpdate()
        IsSelecting = True
        Dim tn As TreeNode
        SetSelection(path, tn)
        Me.EndUpdate()
        IsSelecting = False
        RaiseEvent FolderSelectionChanged(Me, New EventArgs(m_strSelectedPath))
        Me.Focus()
        Me.Cursor = Cursors.Default
    End Sub

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Sub SetSelection(ByVal [path] As String, ByRef trnode As TreeNode)
        Me.Cursor = Cursors.WaitCursor
        [path] = [path].ToUpper()
        Dim strParentDir As String = IO.Path.GetDirectoryName(path)
        Dim i As Integer
        If (Not strParentDir Is Nothing) Then
            SetSelection(strParentDir, trnode)
            If (trnode Is Nothing) Then Return
            For i = 0 To trnode.Nodes.Count - 1
                If DirectCast(trnode.Nodes(i), TreeNode).Path.ToUpper() = [path] Then
                    trnode = DirectCast(trnode.Nodes(i), TreeNode)
                    trnode.Expand() : trnode.EnsureVisible() : SelectedNode = trnode
                    Return
                End If
            Next
            trnode = Nothing
        Else
            Me.CollapseAll()
            For i = 0 To Me.Nodes.Count - 1
                If DirectCast(Me.Nodes(i), TreeNode).Path.ToUpper().StartsWith([path]) Then
                    trnode = DirectCast(Me.Nodes(i), TreeNode)
                    trnode.Expand() : trnode.EnsureVisible() : SelectedNode = trnode
                    Return
                End If
            Next
        End If
        Me.Cursor = Cursors.Default
    End Sub

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Overrides Sub OnBeforeExpand(ByVal e As TreeViewCancelEventArgs)
        MyBase.OnBeforeExpand(e)
        Dim tn As TreeNode = DirectCast(e.Node, TreeNode)
        If DirectCast(tn.Nodes(0), TreeNode).Path = "." Then CreateSubnodes(tn)
    End Sub

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Overrides Sub OnAfterSelect(ByVal e As TreeViewEventArgs)
        MyBase.OnAfterSelect(e)
        If IsSelecting Then Exit Sub
        m_strSelectedPath = DirectCast(e.Node, TreeNode).Path
        RaiseEvent FolderSelectionChanged(Me, New EventArgs(m_strSelectedPath))
    End Sub

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Sub CreateSubnodes(ByVal trnode As TreeNode)
        trnode.Nodes.Clear()
        Dim subdir As String
        Try
            For Each subdir In Directory.GetDirectories(trnode.Path)
                Dim tn As TreeNode
                tn = AddNode(trnode.Nodes, IO.Path.GetFileName(subdir) & SearchInfos(subdir), subdir)
                If Not IO.Path.GetFileName(subdir) = "System Volume Information" Then
                    If Directory.GetDirectories(subdir).Length > 0 Then
                        AddNode(tn.Nodes, ".", ".")
                    End If
                    tn.ImageIndex = Me.m_intImageIndex
                    tn.SelectedImageIndex = Me.m_intSelectedImageIndex
                End If
            Next
        Catch ioex As IOException
        Catch ex As Exception
        End Try
    End Sub
    'Diese Funktion filtert die Ordnerattribute, bzw. stellt fest, ob es sich um ein Spezialfolder
    'handelt, und weist ein eigenes Icon zu.
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Function SearchInfos(ByVal subdir As String) As String
        Dim str As String = "  "
        Dim di As New DirectoryInfo(subdir)
        Dim strAt() As String = Split(Trim(di.Attributes.ToString), ",")
        For i As Integer = 0 To strAt.Length - 1
            Dim s As String = Trim(strAt(i))
            Select Case s
                Case "System"
                    str &= "_system"
                Case "Hidden"
                    str &= "_hidden"
                Case "ReadOnly"
                    str &= "_readonly"
            End Select
        Next
        For ii As Integer = 0 To m_ArrayListSpecialFolder.Count - 1
            Dim strsplit() As String = CStr(m_ArrayListSpecialFolder.Item(ii)).Split(CChar(";"))
            If subdir = strsplit(1) Then
                Select Case strsplit(0)
                    Case "MyMusic"
                        m_intImageIndex = 12
                    Case "MyPictures"
                        m_intImageIndex = 14
                    Case "Desktop"
                        m_intImageIndex = 11
                    Case "Favorites"
                        m_intImageIndex = 16
                    Case "Personal"
                        m_intImageIndex = 17
                    Case "SendTo"
                        m_intImageIndex = 18
                    Case "Recent"
                        m_intImageIndex = 19
                    Case Else
                        m_intImageIndex = 7
                End Select
                Return str
            End If
        Next
        Dim intS As Integer = str.IndexOf("s")
        Dim intH As Integer = str.IndexOf("h")
        If intH > -1 And intS = -1 Then
            m_intImageIndex = 9
        ElseIf intS > -1 Then
            m_intImageIndex = 15
        Else
            m_intImageIndex = 7
        End If
        Return str
    End Function

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Overridable Function AddNode(ByVal nodeCollection As _
             TreeNodeCollection, ByVal caption As String, _
             ByVal path As String) As TreeNode
        Dim tn As New TreeNode
        tn.Text = caption
        tn.Path = path
        nodeCollection.Add(tn)
        Return tn
    End Function
    'Eigene Knoten-Klasse mit Zusatzinformation ber den Pfad
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Class TreeNode
        Inherits System.Windows.Forms.TreeNode
        Public Path As String

    End Class
    'Diese innere Klasse fgt dem Standard-EventArgs eine weiter Information ber den Pfad hinzu.
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Public Class EventArgs
        Inherits System.EventArgs
        Public ReadOnly Path As String

        <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
        Public Sub New(ByVal path As String)
            Me.Path = path
        End Sub

    End Class

End Class


