
'Klaus D. Raudszus
'Raudszusklaus@aol.com

Option Strict On

Imports System.Globalization
Imports System.Security.Permissions
Imports System.Windows.Forms
Imports System.Drawing
Imports System.ComponentModel
Imports System.IO
Imports DirectoryBrowser
Imports FileInformation
Imports FileInformation.Enumeratoren

<PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust"), _
DefaultProperty("FolderBrowser"), _
ToolboxBitmap(GetType(FileBrowser))> _
Public Class FileBrowser
    Inherits ListView

    Private components As System.ComponentModel.IContainer

    Protected m_DirBrowser As DirectoriesBrowser
    Protected m_FileInformation As FileInformation.FileInformation
    Protected m_BrowserFileSort As BrowserFileSort
    Protected m_View As eView = eView.Details
    Protected m_InitColHeader As InitColHeaders

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Public Sub New()
        Me.m_BrowserFileSort = New BrowserFileSort
        InitializeComponent()
        Me.SmallImageList = m_ImgListSmall
        Me.LargeImageList = m_ImgListLarge
        Me.m_InitColHeader = New InitColHeaders(Me)
        Me.m_InitColHeader = Nothing
        Me.View = View.Details
    End Sub

    <Browsable(True), _
    EditorBrowsable(EditorBrowsableState.Always), _
    Category("FileBrowserHandling"), _
    Description("Stellt die Ansicht des FileBrowsers ein.")> _
    Public Property ListViewView() As Enumeratoren.eView
        Get
            Return m_View
        End Get
        Set(ByVal Value As Enumeratoren.eView)
            m_View = Value
            Me.View = CType(Value, View)
        End Set
    End Property

    <Browsable(True), _
    EditorBrowsable(EditorBrowsableState.Always), _
    Category("FileBrowserHandling"), _
    Description("Stellt die Verbindung zum DirectoryBrowser her.")> _
    Public WriteOnly Property FolderBrowser() As DirectoriesBrowser
        Set(ByVal Value As DirectoriesBrowser)
            m_DirBrowser = Value
            AddHandler m_DirBrowser.FolderSelectionChanged, AddressOf FolderSelectionChanged
        End Set
    End Property

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Sub FolderSelectionChanged(ByVal sender As Object, _
                                         ByVal e As DirectoriesBrowser.EventArgs)
        Me.ListViewItemSorter = Nothing
        For Each lvi As ListViewItem In Me.Items
            DirectCast(lvi.Tag, IDisposable).Dispose()
        Next
        Me.Items.Clear()
        m_ImgListSmall.Images.Clear()
        m_ImgListLarge.Images.Clear()
        Dim i As Integer = 0
        Try
            For Each strDir As String In Directory.GetFiles(e.Path)
                m_FileInformation = New FileInformation.FileInformation(strDir)
                m_ImgListSmall.Images.Add(m_FileInformation.FileSmallIcon)
                m_ImgListLarge.Images.Add(m_FileInformation.FileLargeIcon)
                Dim lvi As ListViewItem = Me.Items.Add(m_FileInformation.FileName)
                With lvi
                    .Tag = m_FileInformation
                    .SubItems.Add(m_FileInformation.FileExeType)
                    Dim nf As NumFormat = New NumFormat
                    .SubItems.Add(nf.NumberFormat(m_FileInformation.FileLength.ToString))
                    .SubItems.Add(m_FileInformation.FileType)
                    .SubItems.Add(m_FileInformation.FileLastChanged.ToString)
                    .SubItems.Add(m_FileInformation.FileLastAccess.ToString)
                    .SubItems.Add(m_FileInformation.FileAttributes.ToString)
                    .ImageIndex = i
                End With
                i += 1
                m_FileInformation = Nothing
            Next
        Catch ex As Exception
        Catch ioex As IOException
        End Try
        Me.ListViewItemSorter = m_BrowserFileSort
    End Sub

    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
    Protected Overrides Sub OnColumnClick(ByVal e As ColumnClickEventArgs)
        MyBase.OnColumnClick(e)
        Dim colheader As FBColumnHeader = DirectCast(Me.Columns(e.Column), FBColumnHeader)
        m_BrowserFileSort.SortByCriterion = colheader.Sort
        Me.Sort()
    End Sub

#Region " InitializeComponent "

    Friend WithEvents m_ImgListSmall As System.Windows.Forms.ImageList
    Friend WithEvents m_ImgListLarge As System.Windows.Forms.ImageList

    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.m_ImgListSmall = New System.Windows.Forms.ImageList(Me.components)
        Me.m_ImgListLarge = New System.Windows.Forms.ImageList(Me.components)
        '
        'm_ImgListSmall
        '
        Me.m_ImgListSmall.ImageSize = New System.Drawing.Size(16, 16)
        Me.m_ImgListSmall.TransparentColor = System.Drawing.Color.Transparent
        '
        'm_ImgListLarge
        '
        Me.m_ImgListLarge.ImageSize = New System.Drawing.Size(32, 32)
        Me.m_ImgListLarge.TransparentColor = System.Drawing.Color.Transparent

    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

#End Region

    'Klaus D. Raudszus
    'Raudszusklaus@aol.com

    'Diese innere Klasse erstellt ein Format fr die Zahlen im Header "Gre in Bytes"
    '#.###.### und ist als "Bausatz" bernommen.
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
     Protected Class NumFormat

        <VBFixedString(1)> _
        Const DEC_SEPARATOR As String = ","
        <VBFixedString(1)> _
        Const GROUP_SEPARATOR As String = "."
        <VBFixedString(2)> _
        Const NEG_SIGN As String = "- "

        Const DEC_DIGITS As Integer = 0
        Const GROUP_SIZES As Integer = 3

        <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")> _
        Public Function NumberFormat(ByVal strTxt As String) As String
            Dim nfi As NumberFormatInfo = New NumberFormatInfo
            Dim intSizes As Integer() = {GROUP_SIZES}
            With nfi
                .Clone()
                .NumberDecimalSeparator = DEC_SEPARATOR
                .NumberDecimalDigits = DEC_DIGITS
                .NumberGroupSizes = intSizes
                .NumberGroupSeparator = GROUP_SEPARATOR
                .NegativeSign = NEG_SIGN
            End With
            Dim dblValue As Double = CDbl(strTxt)
            Dim str As String = (dblValue.ToString("N", nfi))
            Return str
        End Function

    End Class

End Class







