
'Klaus D. Raudszus
'Raudszusklaus@aol.com

Option Strict On

Imports System
Imports System.IO
Imports System.Drawing

Public Class FileInformation
    Implements IDisposable

    Public ReadOnly FileName As String
    Public ReadOnly FileType As String
    Public ReadOnly FileLength As Long
    Public ReadOnly FileExeType As String
    Public ReadOnly FileLastChanged As DateTime
    Public ReadOnly FileLastAccess As DateTime
    Public ReadOnly [FileAttributes] As FileAttributes
    Public ReadOnly FileLargeIcon As Bitmap
    Public ReadOnly FileSmallIcon As Bitmap
    'Auslesen der einzelnen Dateien mit den dazugehrigen Informationen. Das
    'Meiste geht ber die Klasse FileInfo aus VB.Net, die Ikons mssen aber
    'ber den Api-Aufruf SHGetFileInfo ermittelt werden.(Siehe Klasse Win32Api)
    Public Sub New(ByVal [path] As String)
        Dim fi As FileInfo = New FileInfo(path)
        Dim strName As String = fi.Name
        Dim intRemove As Integer = strName.LastIndexOf(".")
        Dim strRemove As String = fi.Extension
        If strRemove.StartsWith(".") = True Then _
            strRemove = strRemove.Remove(0, 1)
        Me.FileExeType = strRemove
        Dim intExtLength As Integer = strRemove.Length
        If intRemove > -1 And strName.EndsWith(strRemove) = True Then _
            strName = strName.Remove(intRemove, strRemove.Length + 1)
        Me.FileName = strName
        Me.FileLength = fi.Length
        Me.FileLastChanged = fi.LastWriteTime
        Me.FileLastAccess = fi.LastAccessTime
        Me.[FileAttributes] = fi.Attributes
        Dim shfi As Win32Api.SHFILEINFO
        Win32Api.SHGetFileInfo(path, 0, shfi, Len(shfi), _
            Win32Api.SHGetFileInfoConstants.SHGFI_TYPENAME Or _
            Win32Api.SHGetFileInfoConstants.SHGFI_SMALLICON Or _
            Win32Api.SHGetFileInfoConstants.SHGFI_ICON)
        Me.FileType = shfi.szTypeName
        Dim iptr As New IntPtr(shfi.hIcon)
        Dim intcon As Integer = iptr.ToInt32
        If intcon = 0 Then : Me.FileSmallIcon = TheRessourceIcon()
        Else : Me.FileSmallIcon = Bitmap.FromHicon(iptr)
        End If
        Win32Api.DestroyIcon(iptr)
        Win32Api.SHGetFileInfo(path, 0, shfi, Len(shfi), _
            Win32Api.SHGetFileInfoConstants.SHGFI_LARGEICON Or _
            Win32Api.SHGetFileInfoConstants.SHGFI_ICON)
        iptr = New IntPtr(shfi.hIcon)
        intcon = iptr.ToInt32
        If intcon = 0 Then : Me.FileLargeIcon = TheRessourceIcon()
        Else : Me.FileLargeIcon = Bitmap.FromHicon(iptr)
        End If
        Win32Api.DestroyIcon(iptr)
    End Sub
    'Nicht alle Icons knnen ausgelesen werden, bzw. einige Dateien haben keinen.
    'Hier wird ein "Reserve-Ikon" erstellt aus der eingebetteten Ressource UIkon
    Private Function TheRessourceIcon() As Bitmap
        Dim sra As System.Reflection.Assembly = _
                     System.Reflection.Assembly.GetExecutingAssembly
        Dim streamResIcon As System.IO.Stream = _
                     sra.GetManifestResourceStream("FileInformation.UIkon.bmp")
        Return DirectCast(Bitmap.FromStream(streamResIcon), Bitmap)
    End Function

    Public Sub Dispose() Implements IDisposable.Dispose
        FileLargeIcon.Dispose()
        FileSmallIcon.Dispose()
    End Sub

End Class
